@@ -5,13 +5,26 @@ import base64
 import aiohttp
 
 import requests
+from requests.adapters import HTTPAdapter
 from urllib.parse import quote_plus, urlencode
 
 
+def request(url, method='GET', retry_times=5, *args, **kwargs):
+    with requests.Session() as s:
+        if retry_times:
+            s.mount('https://', HTTPAdapter(pool_connections=1, max_retries=retry_times))
+        response = s.request(
+            url=url, method=method,
+            *args, **kwargs
+        )
+    return response
+
+
 class DingTalkBot:
+    host = 'https://api.dingtalk.com'
     url = 'https://oapi.dingtalk.com/robot/send'
 
-    def __init__(self, token, secret=None, auto_throttle=True):
+    def __init__(self, token, secret=None, app_key=None, app_secret=None, auto_throttle=True):
         self.access_token = token
         self.secret = secret
         self.interval_set = {}
@@ -20,6 +33,44 @@ class DingTalkBot:
         self.report_record = []
         self.wait_seconds = 61
 
+        self.app_key = app_key
+        self.app_secret = app_secret
+        self.app_access_token = None
+        if self.app_key and self.app_secret:
+            self.app_access_token = self.get_app_access_token()
+
+    def get_app_access_token(self):
+        r = request(
+            f'https://api.dingtalk.com/v1.0/oauth2/accessToken',
+            method='POST',
+            json={
+                'appKey': self.app_key,
+                'appSecret': self.app_secret
+            }
+        )
+        return r.json()['accessToken']
+
+    def send_text_to_personal(self, text, phone):
+        if not self.app_access_token:
+            raise RuntimeError('缺少App Access Token')
+        r = request(f'https://oapi.dingtalk.com/user/get_by_mobile', params={
+            'access_token': self.app_access_token,
+            'mobile': phone
+        })
+        user_id = r.json()['userid']
+
+        request(
+            'https://api.dingtalk.com/v1.0/robot/oToMessages/batchSend',
+            method='POST',
+            headers={'x-acs-dingtalk-access-token': self.app_access_token},
+            json={
+                'robotCode': self.app_key,
+                'userIds': [user_id],
+                'msgKey': 'sampleText',
+                'msgParam': f'{{"content": "{text}"}}'
+            }
+        )
+
     @staticmethod
     def get_signature(ts, s):
         secret_enc = s.encode('utf-8')
@@ -66,7 +117,14 @@ class DingTalkBot:
             sign = self.get_signature(timestamp, self.secret)
             params['sign'] = sign
 
-        url = f'{self.url}?{urlencode(params)}'
+        if self.access_token.startswith('https://'):
+            u = self.access_token
+            params.pop('access_token')
+            sep = '&'
+        else:
+            sep = '?'
+            u = self.url
+        url = f'{u}{sep}{urlencode(params)}'
         async with aiohttp.ClientSession() as s:
             timeout = aiohttp.ClientTimeout(sock_read=20, sock_connect=10)
             async with s.request(
@@ -93,8 +151,16 @@ class DingTalkBot:
         if self.secret:
             sign = self.get_signature(timestamp, self.secret)
             params['sign'] = sign
-        url = f'{self.url}?{urlencode(params)}'
-        resp = requests.post(url, json=data, timeout=(10, 20))
+
+        if self.access_token.startswith('https://'):
+            u = self.access_token
+            params.pop('access_token')
+            sep = '&'
+        else:
+            sep = '?'
+            u = self.url
+        url = f'{u}{sep}{urlencode(params)}'
+        resp = request(method='POST', url=url, json=data, timeout=(10, 20))
         if self.check_response(resp):
             return resp.text
 
